@@ -2,7 +2,7 @@
  *  calc_basic.c - arithmetic precedence handling and computing in basic 
  *			calculator mode.
  *	part of galculator
- *  	(c) 2002-2003 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -45,9 +45,7 @@
 static char 	*operator_precedence[] = {"=)", "+-&|x", "*/<>%", "^", "(", "%", NULL};
 static char 	*right_associative = "^";
 
-static s_alg_stack	*current_stack=NULL;
 static GArray		*rpn_stack;
-static GPtrArray	*stackstack;
 static int		rpn_stack_size;
 static int		alg_debug = 0, rpn_debug = 0;
 
@@ -65,20 +63,20 @@ double id (double x)
 
 /* debug_input. debug code: enter tokens on stdin.
  */
-
+/*
 void debug_input ()
 {
 	char		input[20];
 	s_cb_token	current_token;
 	
 	scanf ("%s", input);
-	current_token.number.value = atof (input);
+	current_token.number = atof (input);
 	scanf ("%s", input);
 	current_token.operation = input[0];
-	if (current_token.operation == '(') current_token.number.func = id;
+	current_token.func = NULL;
 	printf ("\t\tdisplay value: %f\n", alg_add_token (current_token));
 }
-
+*/
 /* reduce. TRUE if op1 comes before op2 in a computation.
  */
 
@@ -171,7 +169,7 @@ static s_alg_stack *alg_stack_new (s_cb_token this_token)
 	s_alg_stack	*new_stack;
 	
 	new_stack = (s_alg_stack *) malloc (sizeof(s_alg_stack));
-	new_stack->func = this_token.number.func;
+	new_stack->func = this_token.func;
 	new_stack->number = NULL;
 	new_stack->operation = NULL;
 	new_stack->size = 0;
@@ -187,7 +185,7 @@ static void alg_stack_append (s_alg_stack *stack, s_cb_token token)
 	stack->size++;
 	stack->number = (double *) realloc (stack->number, 
 		stack->size * sizeof(double));
-	stack->number[stack->size-1] = token.number.value;
+	stack->number[stack->size-1] = token.number;
 	stack->operation = (char *) realloc (stack->operation,
 		stack->size * sizeof(char));
 	stack->operation[stack->size-1] = token.operation;
@@ -221,46 +219,74 @@ static double alg_stack_pool (s_alg_stack *stack)
 	return stack->number[stack->size-1];
 }
 
-/* alg_stack_free. the stack destructor.
+/* alg_stack_free. the stack destructor. 
  */
 
 static void alg_stack_free (s_alg_stack *stack)
 {
-	free (stack->number);
-	free (stack->operation);
-	free (stack);
+	if (!stack) {
+		if (!stack->number) free (stack->number);
+		if (!stack->operation) free (stack->operation);
+		free (stack);
+	}
+}
+
+/* alg_stack_free_gfunc. wrapper for alg_stack_free for g_slist_foreach.
+ */
+
+static void alg_stack_free_gfunc (gpointer data, gpointer user_data)
+{
+	s_alg_stack	*stack;
+	
+	stack = data;
+	alg_stack_free (stack);
 }
 
 /* alg_add_token. call this from outside. 
  */
 
-double alg_add_token (s_cb_token this_token)
+double alg_add_token (ALG_OBJECT **alg, s_cb_token this_token)
 {
 	static double	return_value;
+	s_alg_stack	*current_stack;
 	
 	switch (this_token.operation) {
 	case '(':
-		g_ptr_array_add (stackstack, current_stack);
-		if (this_token.number.func == NULL)
-			this_token.number.func = id;
-		current_stack = alg_stack_new (this_token);
+		if (this_token.func == NULL) this_token.func = id;
+		*alg = g_slist_prepend (*alg, alg_stack_new (this_token));
 		break;
 	case ')':
-		if (stackstack->len < 1) break;
+		if (g_slist_length (*alg) < 2) break;
+		current_stack = (s_alg_stack *) (*alg)->data;
 		alg_stack_append (current_stack, this_token);
 		return_value = current_stack->func (
 			alg_stack_pool (current_stack));
+		/* we may specify a function after a bracket enclosed expression
+		 * this is necessary e.g. for factorial. This function is 
+		 * applied after the stack function, so sin(3)! will be (sin(3))!
+		 */
+		if (this_token.func != NULL)
+			return_value = this_token.func(return_value);
 		alg_stack_free (current_stack);
-		current_stack = g_ptr_array_remove_index (stackstack, 
-			stackstack->len - 1);
+		*alg = g_slist_delete_link (*alg, *alg);
 		break;
+	case '=':
+		/* first close all open brackets */
+		while (g_slist_length (*alg) > 1) {
+			this_token.operation = ')';
+			this_token.number = alg_add_token (alg, this_token);
+		}
+		this_token.operation = '=';
+		current_stack = (s_alg_stack *) (*alg)->data;
+		alg_stack_append (current_stack, this_token);
+		return_value = alg_stack_pool (current_stack);
+		alg_free (*alg);
+		*alg = alg_init(alg_debug);
+		break;
 	default:
+		current_stack = (s_alg_stack *) (*alg)->data;
 		alg_stack_append (current_stack, this_token);
 		return_value = alg_stack_pool (current_stack);
-		if (this_token.operation == '=') {
-			alg_free();
-			alg_init(alg_debug);
-		}
 		break;
 	}
 	return return_value;
@@ -269,23 +295,25 @@ double alg_add_token (s_cb_token this_token)
 /* alg_init. use this from outside to initialize everything
  */
 
-void alg_init (int debug_level)
+ALG_OBJECT *alg_init (int debug_level)
 {
 	s_cb_token	token;
 	
 	alg_debug = debug_level;
-	stackstack = g_ptr_array_new ();
-	token.number.func = id;
-	current_stack = alg_stack_new(token);	
+	token.func = id;
+	return g_slist_prepend (NULL, alg_stack_new(token));
 }
 
 /* alg_free. call this from outside to clean up properly
  */
 
-void alg_free ()
+void alg_free (ALG_OBJECT *alg)
 {
-	if (!current_stack) alg_stack_free (current_stack);
-	if (!stackstack) g_ptr_array_free (stackstack, TRUE);
+	if (!alg) {
+		g_slist_foreach (alg, alg_stack_free_gfunc, NULL);
+		g_slist_free (alg);
+		alg = NULL;
+	}
 }
 
 /*
@@ -355,7 +383,7 @@ double rpn_stack_operation (s_cb_token current_token)
 	}
 	/* compute it */
 	return_value = compute_expression (left_hand, current_token.operation, 
-		current_token.number.value);
+		current_token.number);
 	if (rpn_debug > 0) fprintf (stderr, "[%s] RPN stack size is %i.\n", 
 		PROG_NAME, (int)rpn_stack->len);
 	if (rpn_debug > 1) debug_rpn_stack_print();
