@@ -2,7 +2,7 @@
  *  calc_basic.h - arithmetic precedence handling and computing in basic 
  *			calculator mode.
  *	part of galculator
- *  	(c) 2002-2003 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,16 +34,15 @@
 	#define BUG_REPORT	"Please submit a bugreport."
 #endif
 
-#define RPN_FINITE_STACK	3
-#define RPN_INFINITE_STACK	-1
+#define RPN_FINITE_STACK		3
+#define RPN_INFINITE_STACK		-1
+
+#include <glib.h>
+
+typedef GSList ALG_OBJECT;
 
 enum {THIS_LEVEL, LEVEL_UP, LEVEL_DOWN};
 
-typedef union {
-	double		value;
-	double		(*func)(double);
-} u_number;
-
 typedef struct {
 	double		num;		/* numerator */
 	double		denum;		/* denumerator */
@@ -55,7 +54,8 @@ typedef struct {
 } s_complex;
 
 typedef struct {
-	u_number	number;
+	double		number;
+	double		(*func)(double);
 	char		operation;
 } s_cb_token;
 
@@ -65,12 +65,12 @@ typedef struct {
 	char		*operation;
 	int		size;
 } s_alg_stack;
-
+	
 double id (double x);
 
-double alg_add_token (s_cb_token this_token);
-void alg_init (int debug_level);
-void alg_free ();
+double alg_add_token (ALG_OBJECT **alg, s_cb_token this_token);
+ALG_OBJECT *alg_init (int debug_level);
+void alg_free (ALG_OBJECT *alg);
 
 void rpn_init (int size, int debug_level);
 void rpn_stack_set_array (double *values, int length);
