@@ -1,7 +1,7 @@
 /*
  *  callbacks.c - functions to handle GUI events.
  *	part of galculator
- *  	(c) 2002-2003 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "config_file.h"
 #include "callbacks.h"
 #include "ui.h"
+#include "flex_parser.h"
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -99,7 +100,7 @@ on_about_activate                     (GtkMenuItem     *menuitem,
 	gtk_label_set_justify (about_label, GTK_JUSTIFY_CENTER);
 	about_text = g_strdup_printf (_("<span size=\"x-large\" weight=\"bold\">%s v%s</span>\n\
 <span size=\"large\">a GTK 2 based scientific calculator</span>\n\n\
-(c) 2002-2003 by Simon Floery (chimaira@users.sf.net)"), PROG_NAME, VERSION);
+(c) 2002-2004 by Simon Floery (chimaira@users.sf.net)"), PROG_NAME, VERSION);
 	gtk_label_set_markup (about_label, \
 		about_text);
 	gtk_widget_show (about_dialog);
@@ -123,7 +124,12 @@ on_number_button_clicked               (GtkToggleButton  *button,
 {	
 	if (gtk_toggle_button_get_active(button) == FALSE) return;
 	button_activation (button);
-	display_result_add_digit (*(gtk_button_get_label ((GtkButton *)button)));
+	if (current_status.notation == CS_FORMULA)
+		ui_formula_entry_insert (gtk_button_get_label ((GtkButton *)button));
+	else
+		rpn_stack_lift();
+		display_result_add_digit (*(gtk_button_get_label ((GtkButton *)button)));
+	return;
 }
 
 /* this callback is called if a button for doing one of the arithmetic operations plus, minus, 
@@ -131,20 +137,37 @@ on_number_button_clicked               (GtkToggleButton  *button,
  */
 
 void
-on_operation_button_clicked            (GtkToggleButton       *button,
-                                        gpointer         user_data)
+on_operation_button_clicked            (GtkToggleButton       *button,                                        gpointer         user_data)
 {
 	s_cb_token		current_token;
 	double			return_value, *stack;
 	GtkWidget		*tbutton;
+	char 			*operation_string;
 	
 	if (gtk_toggle_button_get_active(button) == FALSE) return;
 	button_activation (button);
+	
+	if (current_status.notation == CS_FORMULA) {
+		if (strcmp (gtk_widget_get_name ((GtkWidget *) button), "button_enter") == 0)
+			ui_formula_entry_activate();
+		/* as long as we don't support string operation ids, we take
+		 * operation char. take this later on:
+		 * else ui_formula_entry_insert (
+		 *	g_object_get_data (G_OBJECT (button), "display_string"));
+		 */
+		else {
+			operation_string = g_strdup_printf ("%c", (int) g_object_get_data (
+				G_OBJECT (button), "operation"));
+			ui_formula_entry_insert (operation_string);
+			g_free(operation_string);
+		}
+		return;
+	}
+	
 	current_token.operation = (int) g_object_get_data (G_OBJECT (button), "operation");
 	/* current number, get it from the display! */
-	if (current_token.operation != '(') 
-		current_token.number.value = display_result_get_double ();
-	else current_token.number.func = NULL;	
+	current_token.number = display_result_get_double ();
+	current_token.func = NULL;
 	/* do inverse left shift is a right shift */
 	if ((current_token.operation == '<') && \
 		(BIT (current_status.fmod, CS_FMOD_FLAG_INV) == 1)) {
@@ -165,10 +188,9 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 		 * in general, a closing bracket is only useful if there were opening
 		 *	brackets.
 		 */
-		
 		if (((current_token.operation == '(') || current_status.allow_arith_op) && \
 			((current_token.operation != ')') || (display_module_bracket_label_update (GET) > 0))) {
-			return_value = alg_add_token (current_token);
+			return_value = alg_add_token (&main_alg, current_token);
 			display_result_set_double (return_value);
 			display_module_arith_label_update (current_token.operation);
 			
@@ -189,24 +211,26 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 	} else if (current_status.notation == CS_RPN) {
 		switch (current_token.operation) {
 		case '=':
-			rpn_stack_push (current_token.number.value);
+			rpn_stack_push (current_token.number);
 			stack = rpn_stack_get (RPN_FINITE_STACK);
 			display_stack_set_yzt_double (stack);
 			free (stack);
-			current_status.rpn_have_result = FALSE;
+			/* ENT is a stack lift disabling button */
+			current_status.rpn_stack_lift_enabled = FALSE;
 			/* display line isn't cleared! */
 			break;
 		default:
-			current_status.rpn_have_result = FALSE;
 			display_result_set_double (rpn_stack_operation (current_token));
 			stack = rpn_stack_get (RPN_FINITE_STACK);
 			display_stack_set_yzt_double (stack);
 			free (stack);
-			current_status.rpn_have_result = TRUE;
+			/* all other operations are stack lift enabling */
+			current_status.rpn_stack_lift_enabled = TRUE;
 		}
 	} else error_message ("on_operation_button_clicked: unknown status");
 
 	current_status.calc_entry_start_new = TRUE;
+	return;
 }
 
 /* this callback is called if a button for a function manipulating the current 
@@ -218,64 +242,25 @@ void
 on_function_button_clicked             (GtkToggleButton	*button,
                                         gpointer user_data)
 {
-	double			(*func[4])(double);
-	gboolean		is_trigonometric;
-	GtkWidget		*tbutton;
+	double		(*func[4])(double);
+	char 		**display_name;
 	
 	if (gtk_toggle_button_get_active(button) == FALSE) return;
 	button_activation (button);
-
+	if (current_status.notation == CS_FORMULA) {
+		display_name = (char **) g_object_get_data (G_OBJECT (button), "display_names");
+		ui_formula_entry_insert (display_name[current_status.fmod]);
+		if (current_status.fmod != 0) ui_relax_fmod_buttons();
+		return;
+	}
 	memcpy (func, g_object_get_data (G_OBJECT (button), "func"), sizeof (func));
-	is_trigonometric = (gboolean) g_object_get_data (G_OBJECT (button), "is_trigonometric");
 	if (!func) error_message ("This button has no function associated with");
-	/* hyperbolic versions of trigonometric functions doesn't have to pay attention to
-		angle base. therefore do it like a normal function
-	 */
-	if (is_trigonometric && (BIT (current_status.fmod, CS_FMOD_FLAG_HYP) == 0)) {
-		if (BIT (current_status.fmod, CS_FMOD_FLAG_INV) == 0) {
-			display_result_set_double (\
-				func[current_status.fmod](display_result_get_rad_angle()));
-		} else {
-			display_result_set_radiant (\
-				func[current_status.fmod](display_result_get_double()));
-		}	
-	} else {
-		display_result_set_double (\
-			func[current_status.fmod](display_result_get_double()));
-	}
+	display_result_set_double (
+		func[current_status.fmod](display_result_get_double()));
 	current_status.calc_entry_start_new = TRUE;	
-	if (current_status.notation == CS_RPN) current_status.rpn_have_result = TRUE;
-	if (current_status.fmod != 0) {
-		tbutton = glade_xml_get_widget (button_box_xml, "button_inv");
-		gtk_toggle_button_set_active ((GtkToggleButton *) tbutton, FALSE);
-		tbutton = glade_xml_get_widget (button_box_xml, "button_hyp");
-		gtk_toggle_button_set_active ((GtkToggleButton *) tbutton, FALSE);
-	}
-}
-
-void constants_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
-{
-	char		*const_value;
-	
-	const_value = user_data;
-	current_status.rpn_have_result = TRUE;
-	display_result_set (const_value);
-	current_status.rpn_have_result = TRUE;
-	current_status.calc_entry_start_new = TRUE;
-}
-
-
-void
-on_constant_button_clicked             (GtkToggleButton       *button,
-                                        gpointer         user_data)
-{
-	GtkWidget		*menu;
-	
-	if (gtk_toggle_button_get_active(button) == FALSE) return;
-	button_activation (button);
-	menu = ui_constants_menu_create(constant, (GCallback)constants_menu_handler);
-	gtk_menu_popup ((GtkMenu *)menu, NULL, NULL, (GtkMenuPositionFunc) position_menu, 
-		button, 0, 0);
+	if (current_status.notation == CS_RPN) 
+		current_status.rpn_stack_lift_enabled = TRUE;
+	if (current_status.fmod != 0) ui_relax_fmod_buttons();
 }
 
 /* tbutton_fmod - these are function modifiers such as INV (inverse) 
@@ -298,12 +283,22 @@ on_gfunc_button_clicked                (GtkToggleButton       *button,
                                         gpointer         user_data)
 {
 	void	(*func)();
+	char 	*display_string;
 	
 	if (gtk_toggle_button_get_active(button) == FALSE) return;
 	button_activation (button);
+	if (current_status.notation == CS_FORMULA) {
+		display_string = g_object_get_data (G_OBJECT (button), "display_string");
+		if (display_string != NULL) {
+			ui_formula_entry_insert (display_string);
+			return;
+		}
+	}
+	if (strcmp(gtk_widget_get_name((GtkWidget *) button), "button_ee") == 0) 
+		rpn_stack_lift();
 	func = g_object_get_data (G_OBJECT (button), "func");
-	if (func) func(button);
-	else error_message ("This button has no function associated with");
+	if (func != NULL) func(button);
+	else error_message ("This button has no general function associated with");
 }
 
 /*
@@ -379,57 +374,50 @@ on_grad_activate                      (GtkMenuItem     *menuitem,
 	display_change_option (CS_GRAD, DISPLAY_OPT_ANGLE);
 }
 
-
 void
 on_ordinary_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	GtkWidget	*w;
-	
 	if (((GtkCheckMenuItem *)menuitem)->active == FALSE) return;
-	/* printf ("pan\n"); */
 	display_change_option (CS_PAN, DISPLAY_OPT_NOTATION);
-	rpn_free ();
-	all_clear ();
-	w = glade_xml_get_widget (button_box_xml, "button_enter");
-	if (w) gtk_button_set_label ((GtkButton *)w, _("="));
-	w = glade_xml_get_widget (button_box_xml, "button_pow");
-	if (w) gtk_button_set_label ((GtkButton *)w, _("x^y"));
-	w = glade_xml_get_widget (button_box_xml, "button_f1");
-	if (w) gtk_button_set_label ((GtkButton *)w, _("("));
-	w = glade_xml_get_widget (button_box_xml, "button_f2");
-	if (w) gtk_button_set_label ((GtkButton *)w, _(")"));
+	set_widget_visibility (main_window_xml, "formula_entry_hbox", FALSE);
+	rpn_free();
+	all_clear();
+	ui_button_set_pan();
 	display_stack_remove();
 	update_dispctrl();
 	/* pixel above/below display result line */
 	display_update_tags ();
 }
 
-
 void
 on_rpn_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	GtkWidget	*w;
-	
 	if (((GtkCheckMenuItem *)menuitem)->active == FALSE) return;
 	display_change_option (CS_RPN, DISPLAY_OPT_NOTATION);
-	alg_free ();
-	all_clear ();
-	w = glade_xml_get_widget (button_box_xml, "button_enter");
-	if (w) gtk_button_set_label ((GtkButton *)w, _("ENT"));
-	w = glade_xml_get_widget (button_box_xml, "button_pow");
-	if (w) gtk_button_set_label ((GtkButton *)w, _("y^x"));
-	w = glade_xml_get_widget (button_box_xml, "button_f1");
-	if (w) gtk_button_set_label ((GtkButton *)w, _("x<>y"));
-	w = glade_xml_get_widget (button_box_xml, "button_f2");
-	if (w) gtk_button_set_label ((GtkButton *)w, _("roll"));
+	set_widget_visibility (main_window_xml, "formula_entry_hbox", FALSE);
+	alg_free(main_alg);
+	all_clear();
+	ui_button_set_rpn();
 	/* stack is created by all_clear */
 	update_dispctrl();
 	/* pixel above/below display result line */
 	display_update_tags ();
 }
 
+void 
+on_form_activate 			(GtkMenuItem     *menuitem,
+					gpointer         user_data)
+{
+	if (((GtkCheckMenuItem *)menuitem)->active == FALSE) return;
+	display_change_option (CS_FORMULA, DISPLAY_OPT_NOTATION);
+	all_clear();
+	ui_button_set_pan();
+	update_dispctrl();
+	set_widget_visibility (main_window_xml, "formula_entry_hbox", TRUE);	
+}
+
 void
 on_display_control_activate (GtkMenuItem     *menuitem,
 			gpointer         user_data)
@@ -583,8 +571,6 @@ on_cut_activate (GtkMenuItem     *menuitem,
 	clear ();
 }
 
-
-
 void
 on_paste_activate (GtkMenuItem     *menuitem,
 			gpointer         user_data)
@@ -593,7 +579,6 @@ on_paste_activate (GtkMenuItem     *menuitem,
 		gtk_clipboard_get (GDK_SELECTION_CLIPBOARD)));
 }
 
-
 void
 on_copy_activate (GtkMenuItem     *menuitem,
 			gpointer         user_data)
@@ -606,6 +591,30 @@ on_copy_activate (GtkMenuItem     *menuitem,
  * Preferences
  */
 
+void user_function_list_selection_changed_cb (GtkTreeSelection *selection, gpointer data)
+{
+	GtkTreeModel 	*model;
+	char 		*string;
+	GtkWidget	*entry;
+	GtkTreeIter 	current_list_iter;
+	
+        if (gtk_tree_selection_get_selected (selection, &model, &current_list_iter))
+        {
+                gtk_tree_model_get (model, &current_list_iter, UFUNC_NAME_COLUMN, &string, -1);
+		entry = glade_xml_get_widget (prefs_xml, "prefs_ufname_entry");
+		gtk_entry_set_text ((GtkEntry *) entry, string);
+                g_free (string);
+		gtk_tree_model_get (model, &current_list_iter, UFUNC_VARIABLE_COLUMN, &string, -1);
+		entry = glade_xml_get_widget (prefs_xml, "prefs_ufvar_entry");
+		gtk_entry_set_text ((GtkEntry *) entry, string);
+                g_free (string);
+		gtk_tree_model_get (model, &current_list_iter, UFUNC_EXPRESSION_COLUMN, &string, -1);
+		entry = glade_xml_get_widget (prefs_xml, "prefs_ufexpr_entry");
+		gtk_entry_set_text ((GtkEntry *) entry, string);
+                g_free (string);
+        }
+}
+
 void const_list_selection_changed_cb (GtkTreeSelection *selection, gpointer data)
 {
         GtkTreeModel 	*model;
@@ -615,20 +624,19 @@ void const_list_selection_changed_cb (GtkTreeSelection *selection, gpointer data
 	
         if (gtk_tree_selection_get_selected (selection, &model, &current_list_iter))
         {
-                gtk_tree_model_get (model, &current_list_iter, NAME_COLUMN, &string, -1);
+                gtk_tree_model_get (model, &current_list_iter, CONST_NAME_COLUMN, &string, -1);
 		entry = glade_xml_get_widget (prefs_xml, "prefs_cname_entry");
 		gtk_entry_set_text ((GtkEntry *) entry, string);
                 g_free (string);
-		gtk_tree_model_get (model, &current_list_iter, VALUE_COLUMN, &string, -1);
+		gtk_tree_model_get (model, &current_list_iter, CONST_VALUE_COLUMN, &string, -1);
 		entry = glade_xml_get_widget (prefs_xml, "prefs_cvalue_entry");
 		gtk_entry_set_text ((GtkEntry *) entry, string);
                 g_free (string);
-		gtk_tree_model_get (model, &current_list_iter, DESC_COLUMN, &string, -1);
+		gtk_tree_model_get (model, &current_list_iter, CONST_DESC_COLUMN, &string, -1);
 		entry = glade_xml_get_widget (prefs_xml, "prefs_cdesc_entry");
 		gtk_entry_set_text ((GtkEntry *) entry, string);
                 g_free (string);
         }
- 
 }
 
 void
@@ -841,8 +849,14 @@ void
 on_show_menubar1_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+	GtkWidget	*menu_item;
+	
 	prefs.show_menu = gtk_check_menu_item_get_active ((GtkCheckMenuItem *) menuitem);;
 	set_widget_visibility (main_window_xml, "menubar", prefs.show_menu);
+	/* in case this cb is called by the right button mouse click menu */
+	menu_item = glade_xml_get_widget (main_window_xml, "show_menubar1");
+	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menu_item, prefs.show_menu);
+
 }
 
 void
@@ -918,25 +932,7 @@ void on_prefs_rem_display_toggled (GtkToggleButton *togglebutton,
 	prefs.rem_display = gtk_toggle_button_get_active (togglebutton);
 	/* only is important when leaving galculator */
 }
-/*
-void on_prefs_def_number_changed (GtkOptionMenu *optionmenu,
-				gpointer user_data)
-{
-	prefs.def_number = gtk_option_menu_get_history (optionmenu);
-}
-										
-void on_prefs_def_angle_changed (GtkOptionMenu *optionmenu,
-				gpointer user_data)
-{
-	prefs.def_angle = gtk_option_menu_get_history (optionmenu);
-}
-										
-void on_prefs_def_notation_changed (GtkOptionMenu *optionmenu,
-				gpointer user_data)
-{
-	prefs.def_notation = gtk_option_menu_get_history (optionmenu);
-}
-*/
+
 void on_prefs_button_width_changed (GtkSpinButton *spinbutton,
 					GtkScrollType arg1,
 					gpointer user_data)
@@ -953,8 +949,80 @@ void on_prefs_button_height_changed (GtkSpinButton *spinbutton,
 	set_all_buttons_size (prefs.button_width, prefs.button_height);
 }
 
+/*
+ * USER FUNCTIONS
+ */
+
+void user_functions_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
+{
+	int 			index;
+	s_flex_parser_result	result;
+	
+	index = (int) user_data;
+	result = compute_user_function (
+		user_function[index].expression, user_function[index].variable,
+		display_result_get());
+	if (!result.error) {
+		display_result_set_double (result.value);
+		current_status.calc_entry_start_new = TRUE;	
+		if (current_status.notation == CS_RPN) 
+			current_status.rpn_stack_lift_enabled = TRUE;
+	} else fprintf (stderr, "[%s] User function %s(%s)=%s returned with an error.\
+Please check the expression string.\n", PROG_NAME, user_function[index].name, 
+user_function[index].variable, user_function[index].expression); 
+}
+
+void
+on_user_function_button_clicked (GtkToggleButton       *button,
+                                        gpointer         user_data)
+{
+	GtkWidget	*menu;
+	
+	if (gtk_toggle_button_get_active(button) == FALSE) return;
+	button_activation (button);
+	menu = ui_user_functions_menu_create(user_function, (GCallback)user_functions_menu_handler);
+	gtk_menu_popup ((GtkMenu *)menu, NULL, NULL, (GtkMenuPositionFunc) position_menu, 
+		button, 0, 0);
+}
+
+/*
+ * CONSTANTS
+ */
+
+void constants_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
+{
+	char		*const_value;
+	
+	/* push current display value */
+	current_status.rpn_stack_lift_enabled = TRUE;
+	rpn_stack_lift();
+	const_value = user_data;
+	display_result_set (const_value);
+	current_status.rpn_stack_lift_enabled = TRUE;
+	current_status.calc_entry_start_new = TRUE;
+}
+
+
+void
+on_constant_button_clicked (GtkToggleButton       *button,
+                                        gpointer         user_data)
+{
+	GtkWidget		*menu;
+	
+	if (gtk_toggle_button_get_active(button) == FALSE) return;
+	button_activation (button);
+	menu = ui_constants_menu_create(constant, (GCallback)constants_menu_handler);
+	gtk_menu_popup ((GtkMenu *)menu, NULL, NULL, (GtkMenuPositionFunc) position_menu, 
+		button, 0, 0);
+}
+
+/*
+ * MEMORY
+ */
+
 void ms_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 {
+	GtkWidget	*button;
 	int		index;
 	
 	index = GPOINTER_TO_INT(user_data);
@@ -964,6 +1032,16 @@ void ms_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 		memory.len++;
 	}
 	memory.data[index] = display_result_get_double();
+	
+	/* at startup, mr and m+ button are disabled as there is nothing
+	 * to show. now, as there is sth, enable them. see also 
+	 * ui.c::ui_main_window_buttons_create
+	 */
+	button = glade_xml_get_widget (button_box_xml, "button_MR");
+	gtk_widget_set_sensitive (button, TRUE);
+	button = glade_xml_get_widget (button_box_xml, "button_Mplus");
+	gtk_widget_set_sensitive (button, TRUE);
+	
 	current_status.calc_entry_start_new = TRUE;
 }
 
@@ -983,10 +1061,13 @@ on_ms_button_clicked (GtkToggleButton       *button,
 void mr_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 {
 	int		index;
-	
+
+	/* current display value on stack */
+	current_status.rpn_stack_lift_enabled = TRUE;
+	rpn_stack_lift();
 	index = GPOINTER_TO_INT(user_data);
 	display_result_set_double(memory.data[index]);
-	current_status.rpn_have_result = TRUE;
+	current_status.rpn_stack_lift_enabled = TRUE;
 	current_status.calc_entry_start_new = TRUE;
 }
 
@@ -1081,6 +1162,111 @@ on_mx_button_clicked             (GtkToggleButton       *button,
 
 }
 
+void on_prefs_ufclear_clicked (GtkButton *button, gpointer user_data)
+{
+	GtkWidget	*entry;
+	
+	entry = glade_xml_get_widget (prefs_xml, "prefs_ufname_entry");
+	gtk_entry_set_text ((GtkEntry *) entry, "");
+        entry = glade_xml_get_widget (prefs_xml, "prefs_ufvar_entry");
+	gtk_entry_set_text ((GtkEntry *) entry, "");
+        entry = glade_xml_get_widget (prefs_xml, "prefs_ufexpr_entry");
+	gtk_entry_set_text ((GtkEntry *) entry, "");
+}
+
+void on_prefs_ufadd_clicked (GtkButton *button, gpointer user_data)
+{
+	GtkWidget		*entry;
+	GtkTreeIter   		iter;
+	int			nr_user_functions;
+	char 			*name, *value, *desc;
+	
+	entry = glade_xml_get_widget (prefs_xml, "prefs_ufname_entry");
+	name = g_strdup (gtk_entry_get_text ((GtkEntry *) entry));
+        entry = glade_xml_get_widget (prefs_xml, "prefs_ufvar_entry");
+	value = g_strdup (gtk_entry_get_text ((GtkEntry *) entry));
+        entry = glade_xml_get_widget (prefs_xml, "prefs_ufexpr_entry");
+	desc = g_strdup (gtk_entry_get_text ((GtkEntry *) entry));
+	
+	if ((strlen(name) == 0) || (strlen(value) == 0) || (strlen(desc) == 0)) {
+		g_free (name);
+		g_free (value);
+		g_free (desc);
+		return;
+	}
+		
+	nr_user_functions = prefs_user_function_store->length;
+	user_function = (s_user_function *) realloc (user_function, (nr_user_functions + 2) * sizeof(s_user_function));
+	user_function[nr_user_functions + 1].name = NULL;
+	
+	user_function[nr_user_functions].name = name;
+	user_function[nr_user_functions].variable = value;
+	user_function[nr_user_functions].expression = desc;
+	
+	gtk_list_store_append (prefs_user_function_store, &iter);	
+	gtk_list_store_set (prefs_user_function_store, &iter, 
+		UFUNC_NAME_COLUMN, user_function[nr_user_functions].name, 
+		UFUNC_VARIABLE_COLUMN, user_function[nr_user_functions].variable, 
+		UFUNC_EXPRESSION_COLUMN, user_function[nr_user_functions].expression, 
+		-1);
+}
+
+void on_prefs_ufdelete_clicked (GtkButton *button, gpointer user_data)
+{
+	GtkTreePath		*path;
+	int			index, counter, nr_user_functions;
+	GtkTreeIter		current_list_iter;
+		
+	if (!gtk_tree_selection_get_selected (gtk_tree_view_get_selection(
+		(GtkTreeView *)glade_xml_get_widget (prefs_xml, "user_function_treeview")),
+		NULL, &current_list_iter)) return;
+	
+	nr_user_functions = prefs_user_function_store->length;
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (prefs_user_function_store), 
+		&current_list_iter);
+	index = *(gtk_tree_path_get_indices (path));
+
+	gtk_list_store_remove (prefs_user_function_store, &current_list_iter);
+	on_prefs_ufclear_clicked (NULL, NULL);
+
+	for (counter = index; counter < (nr_user_functions - 1); counter++)
+		memcpy (&user_function[counter], &user_function[counter+1], sizeof(s_user_function));
+	
+	nr_user_functions--;
+	user_function = (s_user_function *) realloc (user_function, (nr_user_functions + 1) * sizeof(s_user_function));
+	
+	user_function[nr_user_functions].name = NULL;
+}
+
+void on_prefs_ufupdate_clicked (GtkButton *button, gpointer user_data)
+{
+	GtkWidget	*entry;
+	GtkTreePath	*path;
+	int		index;
+	GtkTreeIter 	current_list_iter;
+	
+	if (!gtk_tree_selection_get_selected (gtk_tree_view_get_selection(
+		(GtkTreeView *)glade_xml_get_widget (prefs_xml, "user_function_treeview")),
+		NULL, &current_list_iter)) return;
+	
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (prefs_user_function_store), 
+		&current_list_iter);
+	index = *(gtk_tree_path_get_indices (path));
+	
+	entry = glade_xml_get_widget (prefs_xml, "prefs_ufname_entry");
+	user_function[index].name = g_strdup (gtk_entry_get_text ((GtkEntry *) entry));
+        entry = glade_xml_get_widget (prefs_xml, "prefs_ufvar_entry");
+	user_function[index].variable = g_strdup (gtk_entry_get_text ((GtkEntry *) entry));
+        entry = glade_xml_get_widget (prefs_xml, "prefs_ufexpr_entry");
+	user_function[index].expression = g_strdup (gtk_entry_get_text ((GtkEntry *) entry));
+	
+	gtk_list_store_set (prefs_user_function_store, &current_list_iter, 
+		UFUNC_NAME_COLUMN, user_function[index].name, 
+		UFUNC_VARIABLE_COLUMN, user_function[index].variable, 
+		UFUNC_EXPRESSION_COLUMN, user_function[index].expression, 
+		-1);
+}
+
 void on_prefs_cclear_clicked (GtkButton *button, gpointer user_data)
 {
 	GtkWidget	*entry;
@@ -1114,19 +1300,19 @@ void on_prefs_cadd_clicked (GtkButton *button, gpointer user_data)
 		return;
 	}
 		
-	nr_consts = store->length;
+	nr_consts = prefs_constant_store->length;
 	constant = (s_constant *) realloc (constant, (nr_consts + 2) * sizeof(s_constant));
-	constant[nr_consts + 1].desc = NULL;
+	constant[nr_consts + 1].name = NULL;
 	
 	constant[nr_consts].name = name;
 	constant[nr_consts].value = value;
 	constant[nr_consts].desc = desc;
 	
-	gtk_list_store_append (store, &iter);	
-	gtk_list_store_set (store, &iter, 
-		NAME_COLUMN, constant[nr_consts].name, 
-		VALUE_COLUMN, constant[nr_consts].value, 
-		DESC_COLUMN, constant[nr_consts].desc, 
+	gtk_list_store_append (prefs_constant_store, &iter);	
+	gtk_list_store_set (prefs_constant_store, &iter, 
+		CONST_NAME_COLUMN, constant[nr_consts].name, 
+		CONST_VALUE_COLUMN, constant[nr_consts].value, 
+		CONST_DESC_COLUMN, constant[nr_consts].desc, 
 		-1);
 }
 
@@ -1137,14 +1323,14 @@ void on_prefs_cdelete_clicked (GtkButton *button, gpointer user_data)
 	GtkTreeIter		current_list_iter;
 		
 	if (!gtk_tree_selection_get_selected (gtk_tree_view_get_selection(
-		(GtkTreeView *)glade_xml_get_widget (prefs_xml, "treeview1")),
+		(GtkTreeView *)glade_xml_get_widget (prefs_xml, "constant_treeview")),
 		NULL, &current_list_iter)) return;
 	
-	nr_consts = store->length;
-	path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &current_list_iter);
+	nr_consts = prefs_constant_store->length;
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (prefs_constant_store), &current_list_iter);
 	index = *(gtk_tree_path_get_indices (path));
 
-	gtk_list_store_remove (store, &current_list_iter);
+	gtk_list_store_remove (prefs_constant_store, &current_list_iter);
 	on_prefs_cclear_clicked (NULL, NULL);
 
 	for (counter = index; counter < (nr_consts - 1); counter++)
@@ -1153,7 +1339,7 @@ void on_prefs_cdelete_clicked (GtkButton *button, gpointer user_data)
 	nr_consts--;
 	constant = (s_constant *) realloc (constant, (nr_consts + 1) * sizeof(s_constant));
 	
-	constant[nr_consts].desc = NULL;
+	constant[nr_consts].name = NULL;
 }
 
 void on_prefs_cupdate_clicked (GtkButton *button, gpointer user_data)
@@ -1164,10 +1350,11 @@ void on_prefs_cupdate_clicked (GtkButton *button, gpointer user_data)
 	GtkTreeIter 	current_list_iter;
 	
 	if (!gtk_tree_selection_get_selected (gtk_tree_view_get_selection(
-		(GtkTreeView *)glade_xml_get_widget (prefs_xml, "treeview1")),
+		(GtkTreeView *)glade_xml_get_widget (prefs_xml, "constant_treeview")),
 		NULL, &current_list_iter)) return;
 	
-	path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &current_list_iter);
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (prefs_constant_store), 
+		&current_list_iter);
 	index = *(gtk_tree_path_get_indices (path));
 	
 	entry = glade_xml_get_widget (prefs_xml, "prefs_cname_entry");
@@ -1177,10 +1364,10 @@ void on_prefs_cupdate_clicked (GtkButton *button, gpointer user_data)
         entry = glade_xml_get_widget (prefs_xml, "prefs_cdesc_entry");
 	constant[index].desc = g_strdup (gtk_entry_get_text ((GtkEntry *) entry));
 	
-	gtk_list_store_set (store, &current_list_iter, 
-		NAME_COLUMN, constant[index].name, 
-		VALUE_COLUMN, constant[index].value, 
-		DESC_COLUMN, constant[index].desc, 
+	gtk_list_store_set (prefs_constant_store, &current_list_iter, 
+		CONST_NAME_COLUMN, constant[index].name, 
+		CONST_VALUE_COLUMN, constant[index].value, 
+		CONST_DESC_COLUMN, constant[index].desc, 
 		-1);
 }
 
@@ -1241,6 +1428,138 @@ void on_prefs_bin_length_value_changed (GtkSpinButton *spinbutton,
 	prefs.bin_length = (int) gtk_spin_button_get_value (spinbutton);
 }
 
+void on_prefs_menu_dec_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	gtk_widget_show (glade_xml_get_widget (prefs_xml, "prefs_vbox_dec"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_hex"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_oct"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_bin"));
+}
+			
+void on_prefs_menu_hex_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_dec"));
+	gtk_widget_show (glade_xml_get_widget (prefs_xml, "prefs_vbox_hex"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_oct"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_bin"));
+}
+			
+void on_prefs_menu_oct_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_dec"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_hex"));
+	gtk_widget_show (glade_xml_get_widget (prefs_xml, "prefs_vbox_oct"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_bin"));
+}
+			
+void on_prefs_menu_bin_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_dec"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_hex"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_oct"));
+	gtk_widget_show (glade_xml_get_widget (prefs_xml, "prefs_vbox_bin"));
+}
+
+void on_prefs_dec_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data)
+{
+	prefs.dec_sep = gtk_toggle_button_get_active (togglebutton);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_dec_sep_char_label"), prefs.dec_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_dec_sep_char"), prefs.dec_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_dec_sep_length_label"), prefs.dec_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_dec_sep_length"), prefs.dec_sep);
+	display_result_getset();
+}
+
+void on_prefs_hex_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data)
+{
+	prefs.hex_sep = gtk_toggle_button_get_active (togglebutton);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_hex_sep_char_label"), prefs.hex_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_hex_sep_char"), prefs.hex_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_hex_sep_length_label"), prefs.hex_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_hex_sep_length"), prefs.hex_sep);
+	display_result_getset();
+}
+
+void on_prefs_oct_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data)
+{
+	prefs.oct_sep = gtk_toggle_button_get_active (togglebutton);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_oct_sep_char_label"), prefs.oct_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_oct_sep_char"), prefs.oct_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_oct_sep_length_label"), prefs.oct_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_oct_sep_length"), prefs.oct_sep);
+	display_result_getset();
+}
+
+void on_prefs_bin_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data)
+{
+	prefs.bin_sep = gtk_toggle_button_get_active (togglebutton);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_bin_sep_char_label"), prefs.bin_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_bin_sep_char"), prefs.bin_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_bin_sep_length_label"), prefs.bin_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_bin_sep_length"), prefs.bin_sep);
+	display_result_getset();
+}
+
+void on_prefs_dec_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data)
+{
+	prefs.dec_sep_length = (int) gtk_spin_button_get_value (spinbutton);
+	display_result_getset();
+}
+
+void on_prefs_hex_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data)
+{
+	prefs.hex_sep_length = (int) gtk_spin_button_get_value (spinbutton);
+	display_result_getset();
+}
+
+void on_prefs_oct_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data)
+{
+	prefs.oct_sep_length = (int) gtk_spin_button_get_value (spinbutton);
+	display_result_getset();
+}
+
+void on_prefs_bin_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data)
+{
+	prefs.bin_sep_length = (int) gtk_spin_button_get_value (spinbutton);
+	display_result_getset();
+}
+
+void on_prefs_dec_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data)
+{
+	prefs_sep_char_changed (editable, prefs.dec_sep_char, CS_DEC);
+}
+
+void on_prefs_hex_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data)
+{
+	prefs_sep_char_changed (editable, prefs.hex_sep_char, CS_HEX);
+}
+
+void on_prefs_oct_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data)
+{
+	prefs_sep_char_changed (editable, prefs.oct_sep_char, CS_OCT);
+}
+
+void on_prefs_bin_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data)
+{
+	prefs_sep_char_changed (editable, prefs.bin_sep_char, CS_BIN);
+}
+
 void on_togglebutton_released (GtkToggleButton *togglebutton, 
 					gpointer user_data)
 {
@@ -1272,4 +1591,29 @@ void on_infinite_stack_size_clicked (GtkRadioButton *rb, gpointer user_data)
 	prefs.stack_size = RPN_INFINITE_STACK;
 	rpn_stack_set_size (prefs.stack_size);
 }
+
+gboolean on_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+{
+	GtkWidget	*menu;
+	
+	if (event->button != 3) return FALSE;
+	menu = ui_right_mouse_menu_create ();
+	gtk_menu_popup ((GtkMenu *) menu, NULL, NULL, NULL, NULL, 3, event->time);
+	return FALSE;
+}
+
+void on_formula_entry_activate (GtkEntry *entry, gpointer user_data)
+{
+	s_flex_parser_result 	result;
+	
+	result = flex_parser(gtk_entry_get_text(entry));
+	ui_formula_entry_state (result.error);
+	if (!result.error) display_result_set_double (result.value);
+}
+
+void on_formula_entry_changed (GtkEditable *editable, gpointer user_data)
+{
+	ui_formula_entry_state(FALSE);
+}
+
 /* END */
