@@ -1,7 +1,7 @@
 /*
  *  callbacks.h
  *	part of galculator
- *  	(c) 2002-2003 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -72,7 +72,10 @@ on_ordinary_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 void
 on_rpn_activate                        (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
+                                        gpointer         user_data);					
+void 
+on_form_activate 			(GtkMenuItem     *menuitem,
+					gpointer         user_data);
 void
 on_about_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -153,16 +156,7 @@ void on_prefs_show_menu_toggled (GtkToggleButton *togglebutton,
 
 void on_prefs_rem_display_toggled (GtkToggleButton *togglebutton, 
 					gpointer user_data);
-/*					
-void on_prefs_def_number_changed (GtkOptionMenu *optionmenu,
-					gpointer user_data);
 					
-void on_prefs_def_angle_changed (GtkOptionMenu *optionmenu,
-					gpointer user_data);
-					
-void on_prefs_def_notation_changed (GtkOptionMenu *optionmenu,
-					gpointer user_data);
-*/					
 void on_prefs_button_width_changed (GtkSpinButton *spinbutton,
 					GtkScrollType arg1,
 					gpointer user_data);
@@ -192,6 +186,57 @@ void on_prefs_bin_bits_value_changed (GtkSpinButton *spinbutton,
 void on_prefs_bin_signed_toggled (GtkToggleButton *togglebutton, 
 				gpointer user_data);
 
+void on_prefs_dec_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data);
+
+void on_prefs_hex_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data);
+
+void on_prefs_oct_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data);
+
+void on_prefs_bin_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data);
+
+void on_prefs_bin_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data);
+
+void on_prefs_dec_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data);
+					
+void on_prefs_hex_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data);
+
+void on_prefs_oct_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data);
+
+void on_prefs_dec_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data);
+
+void on_prefs_hex_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data);
+					    
+void on_prefs_oct_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data);
+
+void on_prefs_bin_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data);
+
+void on_prefs_menu_dec_activate (GtkMenuItem     *menuitem,
+			gpointer         user_data);
+			
+void on_prefs_menu_hex_activate (GtkMenuItem     *menuitem,
+			gpointer         user_data);
+			
+void on_prefs_menu_oct_activate (GtkMenuItem     *menuitem,
+			gpointer         user_data);
+			
+void on_prefs_menu_bin_activate (GtkMenuItem     *menuitem,
+			gpointer         user_data);
+
 void const_list_selection_changed_cb (GtkTreeSelection *selection, 
 				gpointer data);
 					
@@ -233,3 +278,19 @@ void on_finite_stack_size_clicked (GtkRadioButton *rb,
 			
 void on_infinite_stack_size_clicked (GtkRadioButton *rb,
 			gpointer user_data);
+
+gboolean on_button_press_event (GtkWidget *widget,
+						GdkEventButton *event,
+						gpointer user_data);
+
+void on_formula_entry_activate (GtkEntry *entry,
+                                            gpointer user_data);
+					    
+void on_formula_entry_changed (GtkEditable *editable, gpointer user_data);
+
+void
+on_user_function_button_clicked (GtkToggleButton       *button,
+                                        gpointer         user_data);
+
+void user_function_list_selection_changed_cb (GtkTreeSelection *selection, 
+					gpointer data);
