@@ -186,7 +186,8 @@ char *add_leading_zeros (char *string, int multiple)
 	int	length, offset, counter;
 	
 	/* I don't want "0" to become "000000000" or whatever */
-	if (strcmp (string, "0") == 0) return string;
+	if (strcmp (string, "0") == 0) return g_strdup(string);
+	if (strcmp (string, MY_INFINITY_STRING) == 0) return g_strdup(string);
 	length = strlen(string);
 	offset = (multiple - length%multiple)%multiple;
 	length += offset;
@@ -220,7 +221,11 @@ void set_checkbutton (GladeXML *xml, char *checkbutton_name, void *is_active)
 	
 	bool_var = is_active;
 	toggle_button = (GtkToggleButton *) glade_xml_get_widget (xml, checkbutton_name);	
-	if (toggle_button) gtk_toggle_button_set_active (toggle_button, *bool_var);
+	if (toggle_button) {
+		gtk_toggle_button_set_active (toggle_button, *bool_var);
+		/* they say there is no good reason. i say there is */
+		gtk_toggle_button_toggled (toggle_button);
+	}
 }
 
 void set_spinbutton (GladeXML *xml, char *spinbutton_name, void *value)
@@ -292,6 +297,16 @@ void set_stacksize (GladeXML *xml, char *name, void *stack_size)
 	gtk_toggle_button_set_active (tb, TRUE);
 }
 
+void set_entry (GladeXML *xml, char *entry_name, void *entry_text)
+{
+	GtkEntry	*entry;
+	char 		**string_var;
+	
+	string_var = entry_text;	
+	entry = (GtkEntry *) glade_xml_get_widget (xml, entry_name);
+	if (entry) gtk_entry_set_text (entry, *string_var);	
+}
+
 /*
  * convert given GdkColor to a string so that gdk_color_parse gives the 
  * same color again.
@@ -397,7 +412,7 @@ char *get_display_number_string (double value, int base)
 					prefs.bin_length);
 			break;
 		default:
-			string_value = _("unknown number base");
+			string_value = g_strdup(_("unknown number base"));
 			fprintf (stderr, _("[%s] unknown number base in function \"get_display_number_string\". %s\n"), PROG_NAME, BUG_REPORT);
 	}
 	return string_value;
@@ -549,6 +564,82 @@ double string2double (char *string, int number_base)
 	return 0;
 }
 
+/* string_separator. insert separator.
+ * 	string - the string to modify
+ *	separate - whether to do sth at all
+ *	block_length - insert separator all block_length chars
+ *	separator - separator char to insert
+ * 	dpoint - char representing the decimal point
+ * we do not free string here as it might be used later on!
+ */
+
+char *string_add_separator (char* string, gboolean separate, int block_length, char separator, char dpoint)
+{
+	int	int_length=0, frac_length=0, counter=0, new_counter=0, offset;
+	char 	*new_string;
+	
+	
+	if (!separate) return g_strdup(string);
+	/* at first, get length of parts pre- and succeeding the decimal point */
+	while ((string[int_length] != '\0') && (string[int_length] != dpoint))
+		int_length++;
+	if (string[int_length] != '\0')
+		while (string[int_length + frac_length + 1] != '\0') frac_length++;
+	/* then allocate memory for new string holding separators */
+	new_string = (char *) malloc ((strlen(string) + (int_length-1)/block_length +
+		(frac_length-1)/block_length + 1) * sizeof(char));
+	/* then copy from string to new_string and insert separators */
+	while ((string[counter] != '\0') && (string[counter] != dpoint)) {
+		/* > ( == ) is horrible, yes, but somehow cool. avoids space
+		 * between sign char '-' and leading digit
+		 */
+		if ((counter > (*string == '-')) && ((int_length % block_length) == (counter % block_length))) {
+			new_string[new_counter] = separator;
+			new_counter++;
+		}
+		new_string[new_counter] = string[counter];
+		new_counter++;
+		counter++;
+	}
+	if (string[int_length] != '\0') {
+		/* copy decimal point */
+		new_string[new_counter] = string[counter];
+		new_counter++;
+		counter++;
+		offset = counter;
+		while (string[counter] != '\0') {
+			if (((counter - offset) > 0) && ((counter - offset) % block_length == 0)) {
+				new_string[new_counter] = separator;
+				new_counter++;
+			}
+			new_string[new_counter] = string[counter];
+			new_counter++;
+			counter++;
+		}
+	}
+	new_string[new_counter] = '\0';
+	return new_string;
+}
+
+/* string_del_separator. removes separator from string, in place.
+ */
+
+char *string_del_separator (char *string, char separator)
+{
+	int	counter=0, new_counter=0;
+	
+	while (string[counter] != '\0') {
+		if (string[counter] == separator) {
+			counter++;
+			continue;
+		}
+		string[new_counter] = string[counter];
+		counter++;
+		new_counter++;
+	}
+	string[new_counter] = '\0';
+	return string;
+}
 
 void set_button_label_and_tooltip (GladeXML *xml, char *button_name, 
 	char *label, char *tooltip)
@@ -567,12 +658,15 @@ void set_button_label_and_tooltip (GladeXML *xml, char *button_name,
 
 gboolean formula_entry_is_active (GtkWidget *window_widget)
 {
-	GtkWidget	*active_widget, *main_window;
+	GtkWidget	*active_widget=NULL, *main_window=NULL;
 	
 	main_window = glade_xml_get_widget (main_window_xml, "main_window");
-	active_widget = gtk_window_get_focus ((GtkWindow *)main_window);
-	return (!strcmp (gtk_widget_get_name (gtk_widget_get_toplevel(window_widget)),"main_window") & 
-		!strcmp (gtk_widget_get_name (active_widget), "formula_entry"));
+	if (main_window != NULL)
+		active_widget = gtk_window_get_focus ((GtkWindow *)main_window);
+	if (active_widget != NULL)
+		return !strcmp (gtk_widget_get_name (gtk_widget_get_toplevel(window_widget)),"main_window") &
+			!strcmp (gtk_widget_get_name (active_widget), "formula_entry");
+	return FALSE;
 }
 
 s_flex_parser_result compute_user_function (char *expression, char *variable, char *value)
@@ -625,3 +719,80 @@ double rad2x (double rad)
 	}
 	return rad;
 }
+
+gboolean get_sep (int number_base)
+{
+	switch (number_base) {
+	case CS_DEC:
+		return prefs.dec_sep;
+	case CS_HEX:
+		return prefs.hex_sep;
+	case CS_OCT:
+		return prefs.oct_sep;
+	case CS_BIN:
+		return prefs.bin_sep;
+	default:
+		fprintf (stderr, _("[%s] unknown number base in function \"get_sep\". %s\n"), PROG_NAME, BUG_REPORT);
+	}
+	return 0;
+}
+
+int get_sep_length (int number_base)
+{
+	switch (number_base) {
+	case CS_DEC:
+		return prefs.dec_sep_length;
+	case CS_HEX:
+		return prefs.hex_sep_length;
+	case CS_OCT:
+		return prefs.oct_sep_length;
+	case CS_BIN:
+		return prefs.bin_sep_length;
+	default:
+		fprintf (stderr, _("[%s] unknown number base in function \"get_sep_length\". %s\n"), PROG_NAME, BUG_REPORT);
+	}
+	return 0;
+}
+
+char get_sep_char (int number_base)
+{
+	switch (number_base) {
+	case CS_DEC:
+		return prefs.dec_sep_char[0];
+	case CS_HEX:
+		return prefs.hex_sep_char[0];
+	case CS_OCT:
+		return prefs.oct_sep_char[0];
+	case CS_BIN:
+		return prefs.bin_sep_char[0];
+	default:
+		fprintf (stderr, _("[%s] unknown number base in function \"get_sep_char\". %s\n"), PROG_NAME, BUG_REPORT);
+	}
+	return 0;
+}
+
+void prefs_sep_char_changed (GtkEditable *editable, char *prefs_sep, int number_base)
+{
+	char 	*sep, *result, **stack;
+	
+	sep = gtk_editable_get_chars (editable, 0, -1);
+	if (strlen(sep) > 0) {
+		if ((is_valid_number(number_base, *sep)) ||
+			((number_base == CS_DEC) && ((*sep == '-') ||
+			(*sep == 'e') || (*sep == dec_point[0]))))
+			gtk_editable_delete_text (editable, 0, -1);
+		else {
+			result = display_result_get();
+			stack = display_stack_get_yzt();
+			if (prefs_sep) g_free (prefs_sep);
+			prefs_sep = g_strdup(sep);
+			if (number_base == current_status.number) {
+				display_result_set(result);
+				display_stack_set_yzt(stack);
+			}
+			g_free (result);
+			g_free (stack);
+		}
+	}
+	g_free (sep);	
+}
