@@ -1,7 +1,7 @@
 /*
  *  general_functions.h
  *	part of galculator
- *  	(c) 2002-2003 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #define _GENERAL_FUNCTIONS_H 1
 
 #include "config_file.h"
+#include "flex_parser.h"
 
 #include <glade/glade.h>
 
@@ -32,6 +33,7 @@ double error_unsupported_inv (double dummy);
 double error_unsupported_hyp (double dummy);
 void error_message (char *message);
 void clear ();
+void backspace ();
 void all_clear ();
 
 double axtof (char *bin_string, int base, int nr_bits, gboolean is_signed);
@@ -47,6 +49,7 @@ void set_spinbutton (GladeXML *xml, char *spinbutton_name, void *value);
 void set_optmenu (GladeXML *xml, char *optmenu_name, void *index);
 void set_button_color (GladeXML *xml, char *button_name, void *color_string);
 void set_stacksize (GladeXML *xml, char *name, void *stack_size);
+void set_entry (GladeXML *xml, char *entry_name, void *entry_text);
 
 char *gdk_color_to_string (GdkColor color);
 
@@ -64,11 +67,29 @@ int get_display_number_length (int base);
 void gfunc_f1 (GtkToggleButton *button);
 void gfunc_f2 (GtkToggleButton *button);
 
-void display_result_changed ();
+void rpn_stack_lift ();
 
 void remember_display_values();
 
-double string2double (char *string);
+double string2double (char *string, int number_base);
+
+char *string_add_separator (char *string, gboolean separate, int block_length, char separator, char dpoint);
+char *string_del_separator (char *string, char separator);
+
+void set_button_label_and_tooltip (GladeXML *xml, char *button_name, 
+	char *label, char *tooltip);
+	
+gboolean formula_entry_is_active (GtkWidget *window_widget);
+
+s_flex_parser_result compute_user_function (char *expression, char *variable, char *value);
+
+double x2rad (double x);
+double rad2x (double x);
+
+gboolean get_sep (int number_base);
+int get_sep_length (int number_base);
+char get_sep_char (int number_base);
+
+void prefs_sep_char_changed (GtkEditable *editable, char *prefs_sep, int number_base);
 
-void update_dispctrl();
 #endif /* general_functions.h */
