@@ -1,7 +1,7 @@
 /*
  *  math_functions.h
  *	part of galculator
- *  	(c) 2002-2003 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,5 +34,11 @@ double grad2rad (double value);
 double asinh (double x);
 double acosh (double x);
 double atanh (double x);
+double sin_wrapper (double x);
+double asin_wrapper (double x);
+double cos_wrapper (double x);
+double acos_wrapper (double x);
+double tan_wrapper (double x);
+double atan_wrapper (double x);
 
 #endif /* math_functions.h */
